@@ -29,8 +29,6 @@
 #include <gtk/gtk.h>
 
 #include "gis-assistant.h"
-#include "gis-assistant-private.h"
-#include "gis-center-container.h"
 
 enum {
   PROP_0,
@@ -49,16 +47,14 @@ static guint signals[LAST_SIGNAL];
 
 struct _GisAssistantPrivate
 {
-  GtkWidget *frame;
   GtkWidget *forward;
   GtkWidget *back;
   GtkWidget *cancel;
   GtkWidget *progress_indicator;
   GtkWidget *main_layout;
-  GtkWidget *action_area;
-  GtkWidget *page_action_widget_area;
   GtkWidget *spinner;
   GtkWidget *titlebar;
+  GtkWidget *stack;
 
   GList *pages;
   GisPage *current_page;
@@ -92,7 +88,9 @@ gis_assistant_switch_to (GisAssistant          *assistant,
                          GisAssistantDirection  direction,
                          GisPage               *page)
 {
-  GIS_ASSISTANT_GET_CLASS (assistant)->switch_to (assistant, direction, page);
+  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
+
+  gtk_stack_set_visible_child (GTK_STACK (priv->stack), GTK_WIDGET (page));
 }
 
 static void
@@ -163,29 +161,6 @@ gis_assistant_previous_page (GisAssistant *assistant)
   gis_assistant_switch_to (assistant, GIS_ASSISTANT_PREV, find_prev_page (priv->current_page));
 }
 
-static void
-remove_from_page_action_area (GtkWidget *widget,
-                              gpointer   user_data)
-{
-  GisAssistantPrivate *priv = user_data;
-  gtk_container_remove (GTK_CONTAINER (priv->page_action_widget_area), widget);
-}
-
-static void
-update_action_widget (GisAssistant *assistant)
-{
-  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
-  GtkWidget *action;
-
-  gtk_container_foreach (GTK_CONTAINER (priv->page_action_widget_area),
-                         remove_from_page_action_area, priv);
-
-  action = gis_page_get_action_widget (priv->current_page);
-  if (action)
-    gtk_container_add (GTK_CONTAINER (priv->page_action_widget_area),
-                       action);
-}
-
 static void
 remove_from_progress_indicator (GtkWidget *widget,
                                 gpointer   user_data)
@@ -265,7 +240,6 @@ update_applying_state (GisAssistant *assistant)
   GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
   if (priv->current_page)
     applying = gis_page_get_applying (priv->current_page);
-  gtk_widget_set_sensitive (priv->frame, !applying);
   gtk_widget_set_sensitive (priv->forward, !applying);
   gtk_widget_set_visible (priv->back, !applying);
   gtk_widget_set_visible (priv->cancel, applying);
@@ -320,7 +294,7 @@ gis_assistant_add_page (GisAssistant *assistant,
   g_signal_connect (page, "destroy", G_CALLBACK (widget_destroyed), assistant);
   g_signal_connect (page, "notify", G_CALLBACK (page_notify), assistant);
 
-  GIS_ASSISTANT_GET_CLASS (assistant)->add_page (assistant, page);
+  gtk_container_add (GTK_CONTAINER (priv->stack), GTK_WIDGET (page));
 
   if (priv->current_page->assistant_priv->link == link->prev)
     update_navigation_buttons (assistant);
@@ -382,16 +356,9 @@ gis_assistant_get_titlebar (GisAssistant *assistant)
   return priv->titlebar;
 }
 
-GtkWidget *
-_gis_assistant_get_frame (GisAssistant *assistant)
-{
-  GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
-  return priv->frame;
-}
-
-void
-_gis_assistant_current_page_changed (GisAssistant *assistant,
-                                     GisPage      *page)
+static void
+update_current_page (GisAssistant *assistant,
+                     GisPage      *page)
 {
   GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
 
@@ -403,12 +370,23 @@ _gis_assistant_current_page_changed (GisAssistant *assistant,
 
   update_titlebar (assistant);
   update_applying_state (assistant);
-  update_action_widget (assistant);
   update_navigation_buttons (assistant);
   update_progress_indicator (assistant);
   gis_page_shown (page);
 }
 
+static void
+current_page_changed (GObject    *gobject,
+                      GParamSpec *pspec,
+                      gpointer    user_data)
+{
+  GisAssistant *assistant = GIS_ASSISTANT (user_data);
+  GtkStack *stack = GTK_STACK (gobject);
+  GtkWidget *new_page = gtk_stack_get_visible_child (stack);
+
+  update_current_page (assistant, GIS_PAGE (new_page));
+}
+
 void
 gis_assistant_locale_changed (GisAssistant *assistant)
 {
@@ -437,63 +415,18 @@ static void
 gis_assistant_init (GisAssistant *assistant)
 {
   GisAssistantPrivate *priv = gis_assistant_get_instance_private (assistant);
-  GtkWidget *widget;
 
-  priv->titlebar = gtk_header_bar_new ();
-  gtk_widget_show (priv->titlebar);
+  gtk_widget_init_template (GTK_WIDGET (assistant));
 
-  priv->main_layout = gtk_box_new (GTK_ORIENTATION_VERTICAL, 20);
-  gtk_box_pack_start (GTK_BOX (assistant), priv->main_layout, TRUE, TRUE, 0);
-
-  priv->frame = gtk_frame_new ("");
-  gtk_frame_set_shadow_type (GTK_FRAME (priv->frame), GTK_SHADOW_NONE);
-  gtk_box_pack_start (GTK_BOX (priv->main_layout), priv->frame, TRUE, TRUE, 0);
-
-  priv->page_action_widget_area = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-
-  priv->forward = gtk_button_new ();
-  gtk_button_set_image (GTK_BUTTON (priv->forward),
-                        gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON));
-  gtk_button_set_use_underline (GTK_BUTTON (priv->forward), TRUE);
-  gtk_widget_set_can_default (priv->forward, TRUE);
-
-  priv->back = gtk_button_new ();
-  gtk_button_set_image (GTK_BUTTON (priv->back),
-                        gtk_image_new_from_stock (GTK_STOCK_GO_BACK, GTK_ICON_SIZE_BUTTON));
-  gtk_button_set_use_underline (GTK_BUTTON (priv->back), TRUE);
-
-  priv->cancel = gtk_button_new ();
-  gtk_button_set_image (GTK_BUTTON (priv->cancel),
-                        gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));
-  gtk_button_set_use_underline (GTK_BUTTON (priv->cancel), TRUE);
-
-  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->titlebar), priv->cancel);
-  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->titlebar), priv->back);
-
-  priv->spinner = gtk_spinner_new ();
-  widget = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-  gtk_alignment_set_padding (GTK_ALIGNMENT (widget), 0, 0, 12, 6);
-  gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->titlebar), widget);
-  gtk_container_add (GTK_CONTAINER (widget), priv->spinner);
-  gtk_widget_show (widget);
-
-  gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->titlebar), priv->forward);
+  g_signal_connect (priv->stack, "notify::visible-child",
+                    G_CALLBACK (current_page_changed), assistant);
 
   g_signal_connect (priv->forward, "clicked", G_CALLBACK (go_forward), assistant);
   g_signal_connect (priv->back, "clicked", G_CALLBACK (go_backward), assistant);
   g_signal_connect (priv->cancel, "clicked", G_CALLBACK (do_cancel), assistant);
 
-  priv->progress_indicator = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_widget_set_halign (priv->progress_indicator, GTK_ALIGN_CENTER);
-
-  priv->action_area = gis_center_container_new (priv->page_action_widget_area,
-                                                priv->progress_indicator);
-
   gis_assistant_locale_changed (assistant);
-  gtk_box_pack_start (GTK_BOX (priv->main_layout), priv->action_area, FALSE, TRUE, 0);
-
   update_applying_state (assistant);
-  gtk_widget_show_all (GTK_WIDGET (assistant));
 }
 
 static void
@@ -521,6 +454,17 @@ gis_assistant_class_init (GisAssistantClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/org/gnome/initial-setup/gis-assistant.ui");
+
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, forward);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, back);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, cancel);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, progress_indicator);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, main_layout);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, spinner);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, titlebar);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisAssistant, stack);
+
   gobject_class->get_property = gis_assistant_get_property;
 
   klass->next_page = gis_assistant_real_next_page;
